#include "Button.h"
#include "../../../../../config.h"
#include "../../../Assets/Texture_Sound.h"
#include "../../../State/ThemeManager.h"

Button::Button(sf::Vector2f pos, sf::Vector2f button_size,
               const std::string& text, size_t font_size,
               std::unique_ptr<ISelectCommand> ptr_command) {

    m_rectangle.setSize(button_size);
    m_rectangle.setPosition(pos);
    m_rectangle.setOutlineColor(ThemeManager::Instance().getButtonOutlineColor());
    m_rectangle.setOutlineThickness(2.f);

    m_text = sf::Text(text, MyAssets_one::GetFont(), font_size);
    m_text.setFillColor(ThemeManager::Instance().getButtonTextColor());
    sf::FloatRect textRect = m_text.getLocalBounds();
    m_text.setPosition(pos.x + (button_size.x - textRect.width) / 2.0f - textRect.left, pos.y + (button_size.y - textRect.height) / 2.0f - textRect.top);
    m_rectangle.setFillColor(ThemeManager::Instance().getButtonFillColor());
    m_ptr_command = std::move(ptr_command);
}

void Button::select() {
    m_rectangle.setFillColor(ThemeManager::Instance().getButtonSelectionColor());
}

void Button::unselect() {
    m_rectangle.setFillColor(ThemeManager::Instance().getButtonFillColor());
}

bool Button::is_selected() const noexcept {
    return m_is_selected;
}

bool Button::is_position_in(sf::Vector2f pos) const noexcept {
    return (m_rectangle.getGlobalBounds().contains(pos));
}
void Button::push() {
    m_ptr_command->execute();
}
void Button::draw_into(sf::RenderWindow& window)  {
    window.draw(m_rectangle);
    window.draw(m_text);
}

void ChooseButton::draw_into(sf::RenderWindow &window) {
    window.draw(m_out_circle);
    if (m_is_selected) window.draw(m_in_circle);
    window.draw(m_text);
}

bool ChooseButton::contains(sf::Vector2f point) const {
    return m_out_circle.getGlobalBounds().contains(point) || m_text.getGlobalBounds().contains(point);
}

ChooseButton::ChooseButton(sf::Vector2f pos, const std::string &text, size_t font_size) {

    m_out_circle.setRadius(10.f);
    m_out_circle.setOutlineThickness(1.f);
    m_out_circle.setOutlineColor(ThemeManager::Instance().getButtonOutlineColor());
    m_out_circle.setFillColor(sf::Color::Transparent);
    m_out_circle.setPosition(pos);

    m_in_circle.setRadius(6.f);
    m_in_circle.setFillColor(sf::Color::Blue);
    m_in_circle.setPosition(pos.x + 4.f, pos.y + 4.f);
    m_in_circle.setOutlineThickness(0);

    m_text = sf::Text(text, MyAssets_one::GetFont(), font_size);
    m_text.setFillColor(ThemeManager::Instance().getButtonTextColor());
    m_text.setPosition(pos.x + 40.f, pos.y - 5.f);
}