#include "IEntity.h"
#include "../../../../config.h"
#include "../Maze/Maze.h"

void IEntity::set_location(Room* ptr_room) {
    m_ptr_room = ptr_room;
}

Food::Food() {
    set_texture(&FoodTexture::instance());
}

std::unique_ptr<IStaticEntity> Food::clone() const {
    return std::make_unique<Food>(*this);
}

void Food::prepare_for_drawing() {
    m_sprite.setTexture(m_ptr_texture->get_texture());
    m_sprite.setPosition(m_ptr_room->get_position()+sf::Vector2f(-10, -10));
}

std::unique_ptr<IGameEvent> Enemy::accept(IVisitor* ptr_visitor) {
    return ptr_visitor->visit(this);
}

std::unique_ptr<IGameEvent> Food::accept(IVisitor* ptr_visitor) {
    return ptr_visitor->visit(this);
}

void Food::draw_into(sf::RenderWindow& window) {
    prepare_for_drawing();
    window.draw(m_sprite);
}

Enemy::Enemy() {
    set_texture(&EnemyTexture::instance());
    m_sprite.setOrigin({config::GAME_ENEMY_SIZE / 2, config::GAME_ENEMY_SIZE / 2});
}

std::unique_ptr<IDynamicEntity> Enemy::clone() const {
    return std::make_unique<Enemy>(*this);
}

void Enemy::prepare_for_drawing() {
    m_sprite.setTexture(m_ptr_texture->get_texture());
    m_sprite.setPosition(m_ptr_room->get_position());
}

void Enemy::draw_into(sf::RenderWindow& window) {
    prepare_for_drawing();
    window.draw(m_sprite);
}

void Enemy::action() {
    auto miliseconds = static_cast<size_t>(m_clock.getElapsedTime().asMilliseconds());
    if (miliseconds < rand() % 10000)
        return;
    auto direction = static_cast<Room::Direction>(rand() % 4);
    m_ptr_room->get_side(direction)->enter(this);
    m_clock.restart();
}