#pragma once

#include "../IDrawable.h"
#include "../../Event/IGameEvent.h"
#include "MyTexture.h"

class Room;
class Food;
class Enemy;

class IVisitor {
public:
    virtual std::unique_ptr<IGameEvent> visit(Food* ptr_entity) = 0;
    virtual std::unique_ptr<IGameEvent> visit(Enemy* ptr_entity) = 0;

};

class IEntity: public IPreparable {
public:
    void set_location(Room* ptr_room);
    Room* get_location() { return m_ptr_room; }
protected:
    Room* m_ptr_room;
};

class IVisitable {
public:
    virtual std::unique_ptr<IGameEvent> accept(IVisitor* ptr_visitor) = 0;
};

struct IStaticEntity : IEntity, IVisitable {
    virtual std::unique_ptr<IStaticEntity> clone() const = 0;
};

class IDynamicEntity: public IEntity, public IVisitable {
public:
    virtual void action() = 0;
    virtual std::unique_ptr<IDynamicEntity> clone() const = 0;
};

class Enemy : public IDynamicEntity {
public:
    Enemy();
    void draw_into(sf::RenderWindow& window) override;
    void action() override;
    void prepare_for_drawing() override;
    void set_texture(MyTexture* ptr_texture)  { m_ptr_texture = ptr_texture; }
    std::unique_ptr<IDynamicEntity> clone() const override;
    std::unique_ptr<IGameEvent> accept(IVisitor* ptr_visitor) override;
    ~Enemy() override {};
private:
    MyTexture* m_ptr_texture;
    sf::Sprite m_sprite;
    sf::Clock m_clock;
};

class Food: public IStaticEntity{
public:
    Food();
    ~Food() override {};
    void draw_into(sf::RenderWindow& window) override;
    void prepare_for_drawing() override;
    void set_texture(MyTexture* ptr_texture) { m_ptr_texture = ptr_texture; }
    std::unique_ptr<IStaticEntity>  clone() const override;
    std::unique_ptr<IGameEvent> accept(IVisitor* ptr_visitor) override;
    bool is_screamer = false;

private:
    MyTexture* m_ptr_texture;
    sf::Sprite m_sprite;
};