#include "Maze.h"
#include "../../../../config.h"
#include "../../State/ThemeManager.h"

Room::Room(float size) : m_rectangle({size, size}){
    m_rectangle.setOrigin(m_rectangle.getSize().x/2,m_rectangle.getSize().y/2 );
    m_rectangle.setFillColor(ThemeManager::Instance().getRoomColor());
}

void Room::set_side(Direction side, std::shared_ptr<IRoomSide> ptr_side) {
    if (side != INVALID) {
        m_sides[side] = std::move(ptr_side);
    }
}

Room::Direction Room::get_direction(IRoomSide* ptr_side) {
    if (m_sides[LEFT].get() == ptr_side) return LEFT;
    if (m_sides[RIGHT].get() == ptr_side) return RIGHT;
    if (m_sides[UP].get() == ptr_side) return UP;
    if (m_sides[DOWN].get() == ptr_side) return DOWN;
    return INVALID;
}

void Room::draw_into(sf::RenderWindow& window) {
    window.draw(m_rectangle);
    for (const auto& side : m_sides) {
        side->draw_into(window);
    }
}

void Maze::draw_into(sf::RenderWindow& m_window) {
    for (auto& room : m_rooms) {
        room->draw_into(m_window);
    }
}

void Wall::prepare_for_drawing() {
    Room::Direction direct = m_room.get_direction(this);
    sf::Vector2f pos = m_room.get_position();
    float size = m_room.get_size();
    std::array<sf::Vector2f, 4> corners = { sf::Vector2f{pos.x - size / 2.f, pos.y - size / 2.f},
                                            sf::Vector2f{pos.x + size / 2.f, pos.y - size / 2.f},
                                            sf::Vector2f{pos.x + size / 2.f, pos.y + size / 2.f},
                                            sf::Vector2f{pos.x - size / 2.f, pos.y + size / 2.f} };

    switch (direct) {
        case Room::LEFT:
            m_line[0] = sf::Vertex(corners[0], ThemeManager::Instance().getWallColor());
            m_line[1] = sf::Vertex(corners[3], ThemeManager::Instance().getWallColor());
            break;
        case Room::RIGHT:
            m_line[0] = sf::Vertex(corners[1], ThemeManager::Instance().getWallColor());
            m_line[1] = sf::Vertex(corners[2], ThemeManager::Instance().getWallColor());
            break;
        case Room::UP:
            m_line[0] = sf::Vertex(corners[0], ThemeManager::Instance().getWallColor());
            m_line[1] = sf::Vertex(corners[1], ThemeManager::Instance().getWallColor());
            break;
        case Room::DOWN:
            m_line[0] = sf::Vertex(corners[2], ThemeManager::Instance().getWallColor());
            m_line[1] = sf::Vertex(corners[3], ThemeManager::Instance().getWallColor());
            break;
        case Room::INVALID:
            throw std::runtime_error("Invalid side");
    }

}

void Wall::draw_into(sf::RenderWindow& m_window) {
    prepare_for_drawing();
    m_window.draw(m_line, 2, sf::Lines);
}


void Pass::enter(IEntity* entity) {
    if (entity->get_location() == &m_room2)
        entity->set_location(&m_room1);
    else
        entity->set_location(&m_room2);
}