#pragma once

#include "../IDrawable.h"
#include "../Entity/IEntity.h"

#include <iostream>

class IRoomSide: public IPreparable {
public:
    virtual void enter(IEntity* entity) = 0;
};

class Room: public IDrawable {
public:
    enum Direction { INVALID = -1, LEFT, RIGHT, UP, DOWN };
    explicit Room(float size);
    float get_size() { return m_rectangle.getSize().x; }
    void set_position(sf::Vector2f pos) { m_rectangle.setPosition(pos); }
    sf::Vector2f get_position() { return m_rectangle.getPosition(); }
    void set_side(Direction side, std::shared_ptr<IRoomSide> ptr_side);
    IRoomSide* get_side(Direction side) { return m_sides[side].get(); }
    Direction get_direction(IRoomSide* ptr_side);
    void draw_into(sf::RenderWindow& m_window) override;

public:
    std::array<std::shared_ptr<IRoomSide>, 4> m_sides;
    sf::RectangleShape m_rectangle;
};

class Maze: public IDrawable {
public:
    Maze() = default;
    explicit Maze(std::vector<Room*>& rooms) : m_rooms(rooms) {};
    void draw_into(sf::RenderWindow& m_window) override;
private:
    std::vector<Room*> m_rooms;
};

///стена, которая хранит в себе информацию о том в какой комнате находится
class Wall: public IRoomSide {
public:
    explicit Wall(Room& room) : m_room(room) {};
    void prepare_for_drawing() override;
    void draw_into(sf::RenderWindow& m_window) override;
    void enter(IEntity* entity) override {};
private:
    Room& m_room;
    sf::Vertex m_line[2];
};

///проход, который знает о двух комнатах, которые он связывает
class Pass: public IRoomSide {
public:
    Pass(Room& room1, Room& room2) : m_room1(room1), m_room2(room2) {};
    void draw_into(sf::RenderWindow& m_window) override {};
    void enter(IEntity* entity) override;
    void prepare_for_drawing() override {};
private:
    Room& m_room1;
    Room& m_room2;
};