#pragma once
#include "IState.h"
#include "../Drawable/Maze/Maze.h"
#include "../Context/Context.h"
#include "../Event/IGameEvent.h"
#include "SFML/Audio/Music.hpp"

class GameState: public IState, public IWindowKeeper {
public:
    GameState(IStateManager& state_manager, const sf::VideoMode& video_mode, const std::string& window_title);
    ~GameState() override;
    void event_handling() override;
    void update() override;
    void render() override;
    bool do_step() override;
    void set_maze(Maze&& maze) {m_maze = std::move(maze); }
    void set_context(GameContext&& context);
    bool process_key_pressed(sf::Keyboard::Key code);

    enum class Music {NONE, INGAME, LOST, WIN};
    Music m_current_music = Music::NONE; ///> текущая музыка
private:
    Maze m_maze;
    ContextManager m_context_manager;
    std::vector<std::unique_ptr<IGameEvent>> m_events;
    sf::Music m_sound_win;
    sf::Music m_sound_ingame;
    sf::Music m_sound_lost;
    sf::Music m_screamer_music;
    float m_delta_time;
};
