#include "SelectState.h"

SelectState::SelectState(IStateManager& state_manager,
                         const sf::VideoMode& video_mode,
                         const std::string& window_title) :
        IState(state_manager), IWindowKeeper(video_mode, window_title), m_menu(state_manager) {
    update_theme();
    set_texture(&BackgroundTexture::instance());
    m_backgorund_sprite.setTexture(m_background->get_texture());
}

void SelectState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<ExitState>(m_state_manager));
            break;
        }
    }
}

void SelectState::update() {
    const sf::Vector2f pos = m_window.mapPixelToCoords(sf::Mouse::getPosition(m_window));
    if (sf::Mouse::isButtonPressed(sf::Mouse::Left)) {
        m_menu.process_mouse(pos, true);
        return;
    }
    m_menu.process_mouse(pos, false);
}
void SelectState::render() {
    update_theme();
    m_window.draw(m_backgorund_sprite);
    m_menu.draw_into(m_window);
    m_window.display();
}

bool SelectState::do_step() {
    event_handling();
    update();
    render();
    return true;
}