#pragma once
#include "../../../config.h"

struct ThemeColors {
    sf::Color button_text;
    sf::Color button_fill;
    sf::Color button_selection;
    sf::Color button_frame;
    sf::Color select_level_background;
    sf::Color game_background_ingame;
    sf::Color game_background_win;
    sf::Color game_background_lost;
    sf::Color game_pacman;
    sf::Color game_pacman_invisible;
    sf::Color game_room;
    sf::Color game_wall;
    sf::Color game_food;
    sf::Color game_enemy;
    sf::Color title_text;
    sf::Color room_color;
    sf::Color wall_color;
};


class ThemeManager {
public:
    enum class Theme { LIGHT, DARK };

    ThemeManager(const ThemeManager&) = delete;
    ThemeManager& operator=(const ThemeManager&) = delete;
    static ThemeManager& Instance();
    void applyTheme() {loadThemeColors(); }
    void set_theme(Theme newTheme);

    sf::Color getButtonFillColor() const;
    sf::Color getButtonOutlineColor() const;
    sf::Color getButtonTextColor() const;
    sf::Color getButtonSelectionColor() const;

    sf::Color getTitleTextColor() const;

    sf::Color getGameBackgroundInGameColor() const;
    sf::Color getGameBackgroundWinColor() const;
    sf::Color getGameBackgroundLostColor() const;
    sf::Color getRoomColor() const;
    sf::Color getWallColor() const;


private:
    Theme m_theme = Theme::LIGHT;
    ThemeColors m_currentColors;
    void loadThemeColors();

    ThemeManager() {set_theme(Theme::LIGHT);};
};
